function t_bb,radiance,lambda,response,emissivity=eps,range=range, $
  t_bb_data=t_bb_data,autorange=autorange,truncate=truncate,spline=spline, $
  verbose=verbose

; calculates temperature (K) corresponding to given radiance (W/m2/sr/um) from
; instrument with response curve response(lambda) (lambda in nm, response
; relative) and optional emissivity eps (default 1) - lookup table calculated
; and returned in structure t_bb_data for future use if multiple calls are
; required. If a temperature range for the lookup table different from the
; default is required, it can be specified using range=[t_min,t_max[,t_step]]
; though the program will attempt to autorange if required.

default_t_min=270. ; minimum lookup table temperature
default_t_max=370. ; maximum lookup table temperature
default_t_step=.1  ; lookup table temperature step
max_n_t=5000       ; maximum number of lookup table entries (for autoranging)
n_non_spline_tags=10 ; number of tags in t_bb_data excluding spline
auto=keyword_set(autorange)
chop=keyword_set(truncate)
spline_set=keyword_set(spline)
verb=keyword_set(verbose)
n_lambda=n_elements(lambda)
n_eps=n_elements(eps)
if n_eps gt 1 && n_eps ne n_elements(radiance) then $
  message,'T_BB: Emissivity must match radiance'
if n_elements(response) ne n_lambda then $
  message,'T_BB: Response must have same number of elements as lambda'
n_range=n_elements(range)
range_set=n_range gt 0
t_bb_data_present=n_elements(t_bb_data) gt 0
if range_set then begin
  if n_range lt 2 then message,'T_BB: Range must have at least 2 elements'
  t_min=range[0]
  t_max=range[1]
  if n_range gt 2 then t_step=range[2] else t_step=default_t_step
endif else if t_bb_data_present then begin
  t_min=t_bb_data.t_min
  t_max=t_bb_data.t_max
  t_step=t_bb_data.t_step
endif else begin
  t_min=default_t_min
  t_max=default_t_max
  t_step=default_t_step
endelse
min_radiance=min(radiance,max=max_radiance)
if t_bb_data_present then begin
  too_low=min_radiance lt t_bb_data.rad_min
  too_high=max_radiance gt t_bb_data.rad_max
  if too_low || too_high then begin
    if auto then begin
      if too_low then begin ; expand downwards
        t_min=t_min-.5*(t_max-t_min)>(.5*t_min)
        t_bb_data_present=0
      endif
      if too_high then begin ; expand upwards
        t_max=t_max+.5*(t_max-t_min)
        t_bb_data_present=0
      endif    
      if verb then print,'Expanding T range to ',t_min,t_max
    endif else if chop then begin ; truncate radiance to table limits
      radiance=radiance>t_bb_data.rad_min<t_bb_data.rad_max
    endif else message,'Lookup table radiance range: '+ $
      string(t_bb_data.rad_min)+string(t_bb_data.rad_max)+', data: '+ $
      string(min_radiance)+string(max_radiance)
  endif
endif
if t_bb_data_present then begin
  if range_set then if t_bb_data.t_min ne t_min || t_bb_data.t_max ne t_max $
    || t_bb_data.t_step ne t_step then t_bb_data_present=0
  if spline_set && n_tags(t_bb_data) le n_non_spline_tags then $
    t_bb_data_present=0 ; splines requested but not found in table
  if t_bb_data.n_lambda ne n_lambda then t_bb_data_present=0 else $
    if max(t_bb_data.lambda ne lambda) || max(t_bb_data.response ne response)$
    then t_bb_data_present=0
  if ~t_bb_data_present then print,'T_BB WARNING: overwriting t_bb_data'
endif
if ~t_bb_data_present then begin ; set up t_bb_data
  repeat begin
    n_temps=floor((t_max-t_min)/t_step)+1
    if auto && n_temps gt max_n_t then begin
      n_temps=max_n_t
      t_step=(t_max-t_min)/(n_temps-1)
      if verb then print,'T_BB WARNING: changing t_step to ',t_step
    endif
    temps=t_min+findgen(n_temps)*t_step
    radiances=l_band(lambda,response,temps)
    rad_min=min(radiances,max=rad_max)
    too_low=min_radiance lt rad_min
    too_high=max_radiance gt rad_max
    if too_low || too_high then begin
      if auto then begin
        if too_low then t_min=t_min-.5*(t_max-t_min)>(.5*t_min) ;expand downward
        if too_high then t_max=t_max+.5*(t_max-t_min) ; expand upward
        if verb then print,'Expanding T range to ',t_min,t_max, $
          ', lookup table radiance range: ',rad_min,rad_max
      endif else if chop then begin ; truncate radiance to table limits
        radiance=radiance>rad_min<rad_max
        too_low=0
        too_high=0
      endif else message,'Lookup table radiance range: '+string(rad_min)+ $
        string(rad_max)+', data: '+string(min_radiance)+string(max_radiance)
    endif
  endrep until ~(too_low || too_high)
  if spline_set then begin
    spline=spl_init(radiances,temps)
    t_bb_data={t_bb_str_spline,n_lambda:n_lambda,lambda:lambda, $
      response:response,t_min:t_min,t_max:t_max,t_step:t_step,temps:temps, $
      radiances:radiances,rad_min:rad_min,rad_max:rad_max,spline:spline}
  endif else begin
    t_bb_data={t_bb_str,n_lambda:n_lambda,lambda:lambda,response:response, $
      t_min:t_min,t_max:t_max,t_step:t_step,temps:temps,radiances:radiances, $
      rad_min:rad_min,rad_max:rad_max}
  endelse
endif
if n_eps gt 0 then begin
  if spline_set then return,spl_interp(t_bb_data.radiances,t_bb_data.temps, $
    t_bb_data.spline,radiance/eps) else $
    return,interpol(t_bb_data.temps,t_bb_data.radiances,radiance/eps)
endif else begin
  if spline_set then return,spl_interp(t_bb_data.radiances,t_bb_data.temps, $
    t_bb_data.spline,radiance) else $
    return,interpol(t_bb_data.temps,t_bb_data.radiances,radiance)
endelse

end
